.features {
  display: flex;
  align-items: center;
  padding: 2rem 0;
  width: 100%;
}

.featureSvg {
  height: 200px;
  width: 200px;
}

/* 画像とテキスト表示のためのスタイル - 上部余白を削除 */
.fullScreenContainer {
  width: 100%;
  display: flex;
  flex-direction: column;
  justify-content: flex-start;
  align-items: center;
  padding: 0;
  margin-top: 0;
}

.fullScreenImageContainer {
  width: 100%;
  display: flex;
  justify-content: center;
  align-items: center;
  padding: 0;
}

.fullScreenImage {
  max-width: 100%;
  width: 100%;
  height: auto;
  object-fit: contain;
}

/* テキスト領域のスタイル */
.textContainer {
  width: 100%;
  max-width: 1200px;
  padding: 2rem;
  text-align: center;
}

.textHeading {
  font-size: 2.5rem;
  margin-bottom: 1rem;
}

.textDescription {
  font-size: 1.2rem;
  line-height: 1.6;
}
