/**
 * Any CSS included here will be global. The classic template
 * bundles Infima by default. Infima is a CSS framework designed to
 * work well for content-centric websites.
 */

/* You can override the default Infima variables here. */
:root {
  --ifm-color-primary: #4e77ee;
  --ifm-color-primary-dark: #4268d1;
  --ifm-color-primary-darker: #3d60be;
  --ifm-color-primary-darkest: #324f9e;
  --ifm-color-primary-light: #648af1;
  --ifm-color-primary-lighter: #7b9df3;
  --ifm-color-primary-lightest: #a7bdf7;

  --ifm-code-font-size: 95%;
  --docusaurus-highlighted-code-line-bg: rgba(78, 119, 238, 0.1);
}


/* For readability concerns, you should choose a lighter palette in dark mode. */
[data-theme='dark'] {
  --ifm-color-primary: #7192f5;
  --ifm-color-primary-dark: #5c7fec;
  --ifm-color-primary-darker: #506fde;
  --ifm-color-primary-darkest: #425fc1;
  --ifm-color-primary-light: #8da7f7;
  --ifm-color-primary-lighter: #aec1fb;
  --ifm-color-primary-lightest: #cfdafb;

  --docusaurus-highlighted-code-line-bg: rgba(120, 160, 255, 0.2);
}
